/************************************************************************/
/*      snapshot.c                                                      */
/*      Version control utility that takes a "snapshot" of the state of */
/*      specified RCS control files for version control of a multiple   */
/*      source system.                                                  */
/*      Carl Douglas Burger, Jr.                                        */
/*      Thu  11-08-1990                                                 */
/*      Borland Turbo C++, ANSI mode                                    */
/*      Usage:  Snapshot should be run in the project directory under   */
/*          which the RCS files are kept (i.e. the parent directory of  */
/*          "RCS")  If the source file is not in the RCS directory,     */
/*          the "-i" switch can be used to identify the source file     */
/*          by using "ident" which doesn't require the original         */
/*          RCS format file.                                            */
/*          "rlog" and "ident" must be in the execution path.           */
/*          For wildcards to be supported, wildargs.obj must be linked  */
/************************************************************************/
static char rcsid[] = "$Id: snapshot.c 1.5.1.1 93/04/20 22:39:29 doug Exp $";
#include <ctype.h>
#include <fcntl.h>
#include <io.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>

static char po_file[65];

/************************************************************************/
void help(void)
{
    fputs("usage: snapshot [ -i ] filenames ...\n", stderr);
    fputs("\tPrints a batch file to stdout that extracts current\n", stderr);
    fputs("\tversions of specified files.\n", stderr);
    fputs("\tWildcards are supported.\n", stderr);
    fputs("\t-i\tuses ident instead of rlog to get version number.\n", stderr);
}   /*  help()  */


/************************************************************************/
/*  NAME:   FILE *popen(char *command)                                  */
/*  PURPOSE Executes the command, saving the standard output to a file  */
/*          and returns a stream handle to this file.                   */
/************************************************************************/
FILE *popen(char *command)
{
    int rc, fd, sout;

    flushall();
    sprintf(po_file, "\\%s", tmpnam(NULL));
    sout = dup(1);                  /* save the stdout file handle */
    if ((fd = open(po_file, O_CREAT | O_TRUNC, S_IFREG | S_IWRITE)) < 0) {
        close(sout);
        return NULL;
    }
    if (dup2(fd, 1) < 0) {          /* force the opened file handle */
        close(sout);                /* to be stdout handle          */
        close(fd);
        unlink(po_file);
        return NULL;
    }
    rc = system(command);           /* this command inherits stdout */
    close(fd);
    if (dup2(sout, 1) < 0)          /* restore the stdout handle  */
        fputs("popen: error in dup2(sout, 1)", stderr);
    close(sout);
    if (rc < 0) {                   /*  error occurred in the command */
        unlink(po_file);
        return NULL;
    }
    return (fopen(po_file, "r"));
}       /*  popen()  */

/************************************************************************/
/*  NAME:   int pclose(FILE *fs)                                        */
/*  PURPOSE closed the stream opened with popen() and erases the        */
/*          temporary file                                              */
/************************************************************************/
int pclose(FILE *fs)
{
    int rc;

    rc = fclose(fs);
    unlink(po_file);                    /* erase the temporary file     */
    flushall();
    return (rc);
}       /*  pclose()  */


/************************************************************************/
/*  NAME:   int readln(char *s, int c, FILE *f)                         */
/*  PURPOSE reads a line of text from a stream and discards the         */
/*          terminating CR character.                                   */
/************************************************************************/
int readln(char *s, int c, FILE *f)
{
    char t[256], *rc;
    int l;

    rc = fgets(t, 256, f);
    if ((rc != NULL) && (s != NULL)) {
	l = strlen(t);
	if ((l > 0) && (t[l - 1] == '\n'))
            t[l - 1] = '\0';            /* strip the CR terminator */
        strncpy(s, t, c);               /* copy the desired length */
	if (l >= c)
            s[c - 1] = '\0';            /* add NUL terminator */
    }
    return (rc == NULL);
}   /*	readln()  */


/****************************************************************************/
/*  NAME:   void processrlog(char *filename)                                */
/*  PURPOSE print the checkout command for "filename" using rlog to get     */
/*          the version number                                              */
/****************************************************************************/
void processrlog(char *filename)
{
    FILE *infile;
    char cmdlin[80], textline[256], version[20], desc[128];

    /* build command line to run rlog */
    sprintf(cmdlin, "rlog -t %s", filename);

    if ((infile = popen(cmdlin)) != NULL) {
        version[0] = '\0';

        /* use filename for default description */
        strcpy(desc, filename);
        while (!readln(textline, 256, infile)) {

            /* is this the head revision number? */
            if (strncmp(textline, "head:", 5) == 0) {

                /* grab latest revision number and remove spaces */
                strcpy(version, textline + 5);
                while (isspace(version[0]))
                    strcpy(version, version + 1);
            } else if (strncmp(textline, "description:", 12)== 0) {

                /* read the description line */
                if (!readln(textline, 256, infile))
                    strcpy(desc, textline);
            }
        }
        pclose(infile);

        /* if we found a version number, print out the checkout command */
        if (strlen(version) > 0) {
            printf("REM %s\n", desc);
            printf("co -r%s %s\n", version, strlwr(filename));
        }
    } else
        perror(cmdlin);
}   /*  processrlog()  */


/****************************************************************************/
/*  NAME:   void processident(char *filename)                               */
/*  PURPOSE print checkout command for "filename" using "ident" to get the  */
/*          version num                                                     */
/****************************************************************************/
void processident(char *filename)
{
    FILE *infile;
    char cmdlin[80], textline[256], version[20], *s;
    int i;

    /* build command line to run ident */
    sprintf(cmdlin, "ident -q %s", filename);
    if ((infile = popen(cmdlin)) != NULL) {
        version[0] = '\0';
        while (!readln(textline, 256, infile)) {

            /* look for $Id marker */
            if ((s = strstr(textline, "\x24Id:")) != NULL) {
                s += 5;                 /* skip past $Id string */
                while (!isspace(*s))    /* skip the filename */
                    s++;
                while (isspace(*s))     /* skip the white space */
                    s++;
                if (isdigit(*s)) {
                    i = 0;
                    while (!isspace(*s))    /* fill in the version number */
                        version[i++] = *s++;
                    version[i] = '\0';
                }

                /* print warning if file is locked */
                if (strstr(textline, "Locker") != NULL)
                    fprintf(stderr, "WARNING: %s is locked\n", filename);
            }
        }
        pclose(infile);
        if (strlen(version) > 0) {
            printf("REM %s\n", filename);
            printf("co -r%s %s\n", version, strlwr(filename));
        } else {
            printf("REM %s has no ID\n", filename);
            fprintf(stderr, "WARNING: %s has no ID\n", filename);
        }
    } else
        perror(cmdlin);
}   /*  processident()  */


/************************************************************************/
int main(int argc, char *argv[])
{
    int argidx, ident;

    ident = 0;
    argidx = 1;
    while ((argidx < argc) && (argv[argidx][0] == '-')) {
        switch(argv[argidx][1]) {
            case 'i':
                ident = 1;
                break;
            default:
                help();
                return 1;
        }
        argidx++;
    }
    printf("REM $Id""$\n");     /* "" stops MKS from expanding this keyword */
    printf("REM\n");

    while (argidx < argc) {
        if (ident)
            processident(argv[argidx]);
        else
            processrlog(argv[argidx]);
        argidx++;
    }
    return 0;
}   /*  snapshot.c  */
